/*
 *	Abstraction Layer to the posix socket interface
 *	in kernelspace
 */



#include "socket.h"



/*
 *	Create a communication endpoint (a socket) from kernelspace
 */

struct socket* socket_create( int domain, int type, int protocol  )
{
	struct socket* sock = NULL;


	if( sock_create( domain, type, protocol, &sock ) < 0 ) {
		return NULL;
	}

	return sock;
}



/*
 *	Open a connection to the server from kernelspace
  */

int socket_connect( struct socket* sock, struct sockaddr* address, int addrlen )
{
	return sock->ops->connect( sock, address, addrlen, 0 );
}


/*
 *	Terminate a connection from kernelspace
 */

int socket_close( struct socket* sock )
{
	sock_release( sock );
	return 0;
}



/*
 *	Receive message from the server from kernelspace
 */

ssize_t socket_read( struct socket* sock, void* buffer, size_t count )
{
	struct iovec iov;
	struct msghdr msg;
	struct scm_cookie scm;
	mm_segment_t fs;
	int size = 0;


	fs = get_fs();
	set_fs( get_ds() );

	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;
	msg.msg_control = NULL;

	iov.iov_base = buffer;
	iov.iov_len = count;

	memset( &scm, 0, sizeof( scm ) );

	if( ( size = sock->ops->recvmsg( sock, &msg, count, 0, &scm ) ) >=0 ) {
		scm_recv( sock, &msg, &scm, 0 );
	}

	set_fs( fs );
	return size;
}



/*
 *	Send message to server from kernelspace
 */

ssize_t socket_write( struct socket* sock, void* buffer, size_t count )
{
	struct iovec iov;
	struct msghdr msg;
	struct scm_cookie scm;
	mm_segment_t fs;
	int size = 0;


	fs = get_fs();
	set_fs( get_ds() );

	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;
	msg.msg_control = NULL;
	msg.msg_controllen = 0;
	msg.msg_flags = 0;

	iov.iov_base = (void*) buffer;
	iov.iov_len = count;

	if( ( size = scm_send( sock, &msg, &scm ) ) >= 0 ) {
		size = sock->ops->sendmsg( sock, &msg, count, &scm );
		scm_destroy( &scm );
	}

	set_fs( fs );
	return size;
}
