/*
 *	Abstraction Layer to the posix socket interface
 *	in kernelspace
 */


#include <linux/version.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/socket.h>
#include <linux/fcntl.h>
#include <linux/in.h>
#include <linux/net.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/ctype.h>
#include <linux/locks.h>
#include <linux/time.h>

#include <net/scm.h>
#include <net/ip.h>

#include <asm/uaccess.h>



#ifndef SOCKET_H
#define SOCKET_H

/*
 *	Abstract functions for kernelspace networking stuff
 */


/*
 *	Create a communication endpoint
 *
 *	domain: used lowlevel protocol, e.g. PF_INET for IPv4
 *	type: type of transport protocol, e.g SOCK_STREAM for TCP
 *	protocol: specific protocol (default: 0)
 *
 * Returns a struct gsocket on success and NULL on error
 */

struct socket* socket_create( int domain, int type, int protocol );


/*
 *	Open a connection to the server
 *
 *	sock: allocated struct from socket_create
 *	servaddr: host information specific to to protocol
 *	addrlen: size of servaddr
 *
 *	Returns 0 on success and < 0 on error
 */

int socket_connect( struct socket* sock, struct sockaddr* servaddr, int addrlen );


/*
 *	Terminate a connection
 *
 *	sock: allocated struct from socket_create
 *
 *	Returns 0 on success and -1 on error
 */

int socket_close( struct socket* sock );


/*
 *	Receive a message from the server
 *
 *	sock: allocated struct from socket_create
 *	buffer: byte array where the message should be copied to
 *	count: size of the buffer array
 *
 *	Returns the number of read bytes on success and -1 on error
 */

ssize_t socket_read( struct socket* sock, void* buffer, size_t count );


/*
 *	Send a message to the server
 *
 *	sock: allocated struct from socket_create
 *	buffer: byte array which holds the message
 *	count: size of the buffer array
 *
 *	Returns the number of writtten bytes on success and -1 on error
 */

ssize_t socket_write( struct socket* sock, void* buffer, size_t count );

#endif
