/*
 *	Layer to the ssh implementation
 *	Currently only used to implement the packet protocol for the client sifsd
 */



#include "socket.h"
#include "getput.h"



#ifndef SSH_H
#define SSH_H


#define SSH_BUFSIZE 8192

/*
 *	Common struct for kernellevel access to network functions
 */

struct sshsocket {
	size_t avail;
	struct socket* socket;
};


/*
 *	Client sifsd packet protocol interface
 */


/*
 *	Create a communication endpoint
 *
 *	domain: used lowlevel protocol, e.g. PF_INET for IPv4
 *	type: type of transport protocol, e.g SOCK_STREAM for TCP
 *	protocol: specific protocol (default: 0)
 *
 * Returns a new struct sshsocket on success or NULL in case of an error
 */

struct sshsocket* ssh_create( int domain, int type, int protocol );


/*
 *	Open a connection to the server
 *
 *	sock: allocated struct from ssh_create
 *	servaddr: host information specific to to protocol
 *	addrlen: size of servaddr
 *	user: name of the user who authenticates to the server
 *	passwd: user password
 *
 *	Returns 0 on success or a negative error code
 */

int ssh_connect( struct sshsocket* sock, struct sockaddr* servaddr, int addrlen, char* user, char* passwd );


/*
 *	Send a disconnect and terminate signal to sifsd
 *
 *	sock: allocated struct from ssh_create
 *
 *	Returns 0 on success or a negative error code
 */

int ssh_disconnect( struct sshsocket* sock );


/*
 *	Close a connection from kernelspace to sifsd
 *
 *	sock: allocated struct from ssh_create
 *
 *	Returns 0 on success or a negative error code
 */

int ssh_close( struct sshsocket* sock );


/*
 *	Receive a message from the server via sifsd
 *
 *	sock: allocated struct from ssh_create
 *	buffer: byte array where the message should be copied to
 *	count: size of the buffer array
 *
 *	Returns the number of read bytes on success or a negative error code
 */

ssize_t ssh_read( struct sshsocket* sock, void* buffer, size_t count );


/*
 *	Send a message to the server via sifsd
 *
 *	sock: allocated struct from ssh_create
 *	buffer: byte array which holds the message
 *	count: size of the buffer array
 *
 *	Returns the number of writtten bytes on success or a negative error code
 */

ssize_t ssh_write( struct sshsocket* sock, void* buffer, size_t count );

#endif
