/*
 * Maps a path to a file descriptor
 */



#include "desc.h"


#include <stdio.h>


struct filedesc* hasharray[256] = { 0 };



u_int8_t compute_hash( char* path )
{
	int i = 0;
	int len = strlen( path );
	int cnt = len >> 2;
	u_int8_t hash = 0;
	u_int32_t hash4 = 0;
	u_int32_t* pptr = (u_int32_t*) path;

fprintf( stderr, "compute_hash(): try computing\n" );

	for( i = 0; i < cnt; i++ ) {
		hash4 ^= pptr[i];
	}

	hash = ((u_int8_t*) &hash4)[0] ^ ((u_int8_t*) &hash4)[1] ^ ((u_int8_t*) &hash4)[2] ^ ((u_int8_t*) &hash4)[3];

	for( i = cnt << 2; i < len; i++ ) {
		hash ^= path[i];
	}

fprintf( stderr, "compute_hash(): successful\n" );
	return hash;
}



int desc_insert( char* path, int fd )
{
	u_int8_t hash = 0;
	struct filedesc* current = NULL;


	if( path == NULL ) {
		fprintf( stderr, "desc_insert(): Invalid path string\n" );
		return -1;
	}

	hash = compute_hash( path );
	current = hasharray[hash];

	while( current != NULL ) {
		current = current->next;
	}

	if( ( current = (struct filedesc*) malloc( sizeof( struct filedesc ) ) ) == NULL ) {
		return -1;
	}

	if( ( current->path = (char*) malloc( strlen( path ) + 1 ) ) == NULL ) {
		free( current );
		current = NULL;
		return -1;
	}

	memcpy( current->path, path, strlen( path ) + 1 );
	current->fd = fd;
	current->counter = 0;
	current->next = NULL;
	hasharray[hash] = current;

	return 0;
}



int desc_remove( char* path )
{
	u_int8_t hash = 0;
	struct filedesc* current = NULL;
	struct filedesc* prev = NULL;


	if( path == NULL ) {
		fprintf( stderr, "desc_remove(): Invalid path string\n" );
		return -1;
	}

	hash = compute_hash( path );
	prev = current = hasharray[hash];

fprintf( stderr, "desc_remove(): find node\n" );
	while( current != NULL && strcmp( current->path, path ) != 0 ) {
		prev = current;
		current = current->next;
	}

	if( current == NULL ) {
fprintf( stderr, "desc_remove(): node not found\n" );
		return -1;
	}
fprintf( stderr, "desc_remove(): node found\n" );

	prev->next = NULL;
	free( current->path );
	free( current );

	return 0;
}



int desc_increase( char* path )
{
	u_int8_t hash = 0;
	struct filedesc* current = NULL;


	if( path == NULL ) {
		fprintf( stderr, "desc_increase(): Invalid path string\n" );
		return -1;
	}

	hash = compute_hash( path );
	current = hasharray[hash];

	while( current != NULL && strcmp( current->path, path ) != 0 ) {
		current = current->next;
	}

	if( current == NULL ) {
		return -1;
	}

	current->counter++;
	return 0;
}



int desc_decrease( char* path )
{
	u_int8_t hash = 0;
	struct filedesc* current = NULL;


	if( path == NULL ) {
		fprintf( stderr, "desc_decrease(): Invalid path string\n" );
		return -1;
	}
fprintf( stderr, "desc_decrease(): find node\n" );
fflush( stderr );
	hash = compute_hash( path );
	current = hasharray[hash];

	while( current != NULL && strcmp( current->path, path ) != 0 ) {
		current = current->next;
	}

	if( current == NULL ) {
fprintf( stderr, "desc_remove(): node not found\n" );
		return -1;
	}
fprintf( stderr, "desc_remove(): node found\n" );

	if( current->counter-- == 1 ) {
		return 1;
	}

	return 0;
}



int desc_get( char* path )
{
	u_int8_t hash = 0;
	struct filedesc* current = NULL;


	if( path == NULL ) {
		fprintf( stderr, "desc_get(): Invalid path string\n" );
		return -1;
	}

	hash = compute_hash( path );
	current = hasharray[hash];

	while( current != NULL && strcmp( current->path, path ) != 0 ) {
		current = current->next;
	}

	if( current == NULL ) {
		return -1;
	}

	return current->fd;
}
