/*
 * Returns various file attributes
 */



#include "sifs-process.h"



int sifs_getattr( struct connection* conn )
{
	int len = 0;
	int size = 0;
	struct stat statbuf;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int32_t  packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_getattr: Retrieve inode attributes\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1028 ) {   // one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_getattr: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_getattr: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_getattr: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_getattr: Receiving request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_getattr: Receiving request payload succeeded\n" );

	if( conn->mntpath[0] == 0 ) {   // No filesystem is mounted
		SIFS_DEBUG( "sifs_getattr: There was no mount request before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_getattr: Working directory is correct\n" );

	if( GET_16BIT( conn->in.buffer ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_getattr: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_getattr: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer + 2, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_getattr: Decoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_getattr: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	if( lstat( path, &statbuf ) < 0 ) {
		SIFS_DEBUG( "sifs_getattr: lstat syscall failed\n" );
		if( errno == ENAMETOOLONG ) {
			status = SIFS_ENAMETOOLONG;
			goto out;
		}

		if( errno == ENOENT || errno == ENOTDIR ) {
			status = SIFS_ENOENT;
			goto out;
		}

		if( errno == EACCES ) {
			status = SIFS_EACCES;
			goto out;
		}

		status = SIFS_ESERVER;
		goto out;
	}
SIFS_PROFILE( "sifs_getattr: lstat syscall succeeded\n" );

	PUT_32BIT( conn->out.buffer + 12, statbuf.st_mode );
	PUT_32BIT( conn->out.buffer + 16, statbuf.st_nlink );
	PUT_32BIT( conn->out.buffer + 20, statbuf.st_uid );
	PUT_32BIT( conn->out.buffer + 24, statbuf.st_gid );
	PUT_32BIT( conn->out.buffer + 28, statbuf.st_atime );
	PUT_32BIT( conn->out.buffer + 32, statbuf.st_mtime );
	PUT_32BIT( conn->out.buffer + 36, statbuf.st_ctime );
	PUT_64BIT( conn->out.buffer + 40, (long long) statbuf.st_size );

	size = 36;

out:
	PUT_32BIT( conn->out.buffer + 4, size );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, size + 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_getattr: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_getattr: Sending reply to client succeeded\n" );

	return 0;
}
