/*
 * Helper functions
 */



#include "sifs-process.h"



int sifs_invalidcmd( struct connection* conn )
{
	u_int32_t packetlen = GET_32BIT( conn->out.buffer + 4 );


SIFS_PROFILE( "sifs_invalidcmd: Got invalid request opcode\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );
	PUT_32BIT( conn->out.buffer + 4, 12 );
	PUT_32BIT( conn->out.buffer + 8, SIFS_EPROTO );

	if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_invalidcmd: Unable to consume packet\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_invalidcmd: Receiving request payload succeeded\n" );

	if( atomicio( write, conn->out.fd, conn->out.buffer, 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_release: Sending reply to client succeeded\n" );

	return 0;
}



ssize_t not_implemented( struct connection* conn )
{
	ssize_t size = 0;
	u_int32_t packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "not_implemented: Got request for unimplemented operation\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );
	PUT_32BIT( conn->out.buffer + 4, 12 );
	PUT_32BIT( conn->out.buffer + 8, EPERM );

	if( ( size = consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) ) < 0 ) {
		SIFS_DEBUG( "sifs_invalidcmd: Unable to consume packet\n" );
		return size;
	}
SIFS_PROFILE( "sifs_invalidcmd: Receiving request payload succeeded\n" );

	if( ( size = atomicio( write, conn->out.fd, conn->out.buffer, 12 ) ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Sending reply to client failed\n" );
		return size;
	}
SIFS_PROFILE( "sifs_release: Sending reply to client succeeded\n" );

	return size;
}



ssize_t consume_packet( int fd, u_int8_t* buffer, size_t bufsize, size_t packetlen )
{
	size_t avail = 0;
	ssize_t rbytes = 0;
	size_t consume = 0;


	while( consume < packetlen ) {
		avail = packetlen;
		if( packetlen > bufsize ) {
			avail = bufsize;
		}

		if( ( rbytes = atomicio( read, fd, buffer, avail ) ) < 0 ) {
			return rbytes;
		}
		consume += (size_t) rbytes;
	}

	return consume;
}
