/*
 * Prepare to export a filesystem to the client
 */



#include "sifs-process.h"



int sifs_mount( struct connection* conn )
{
	int len = 0;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int32_t packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_mount: Change into requested directory:\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1032 ) {   // version + one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_mount: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_mount: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_mount: Receiving request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_mount: Receiving request payload succeeded\n" );

	if( conn->mntpath[0] != 0 ) {   // A filesystem is already mounted
		SIFS_DEBUG( "sifs_mount: A filesystem was mounted before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: Request was first request\n" );

	if( GET_32BIT( conn->in.buffer ) != 1 ) {   // Invalid protocol version
		SIFS_DEBUG( "sifs_mount: Invalid protocol version\n" );
		status = SIFS_ENOTSUP;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: Protocol version is correct\n" );

	if( GET_16BIT( conn->in.buffer + 4 ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_mount: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer + 6, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_mount: Decoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	memcpy( conn->mntpath, path, len );

	if( chdir( path ) < 0 ) {
		SIFS_DEBUG( "sifs_mount: chdir syscall failed\n" );
		if( errno == ENAMETOOLONG ) {
			status = SIFS_ENAMETOOLONG;
			goto out;
		}

		if( errno == ENOENT || errno == ENOTDIR ) {
			status = SIFS_ENOENT;
			goto out;
		}

		if( errno == EACCES ) {
			status = SIFS_EACCES;
			goto out;
		}

		status = SIFS_ESERVER;
		goto out;
	}
SIFS_PROFILE( "sifs_mount: chdir syscall succeeded\n" );

	PUT_32BIT( conn->out.buffer + 12, getuid() );
	PUT_32BIT( conn->out.buffer + 16, getgid() );

out:
	PUT_32BIT( conn->out.buffer + 4, 8 );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, 20 ) < 0 ) {
		SIFS_DEBUG( "sifs_mount: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_mount: Sending reply to client succeeded\n" );

	return 0;
}
