/*
 * Opens a file and stores the file descriptor
 */



#include "sifs-process.h"



int sifs_open( struct connection* conn )
{
	int fd = 0;
	int len = 0;
	int flags = 0;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int32_t  packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_open: Open file:\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1036 ) {   // one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_open: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_open: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_open: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_open: Reading request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_open: Receiving request payload failed\n" );

	if( conn->mntpath[0] == 0 ) {   // No filesystem is mounted
		SIFS_DEBUG( "sifs_open: There was no mount request before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_open: Working directory is correct\n" );

	flags = GET_32BIT( conn->in.buffer + 4 ) & 014003;

	if( GET_16BIT( conn->in.buffer + 8 ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_open: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_open: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer + 10, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_open: Decoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_open: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	if( ( fd = desc_get( path ) ) < 0 ) {
SIFS_PROFILE( "sifs_open: File was not opened before\n" );

		if( ( fd = open( path, flags ) ) < 0 ) {
			SIFS_DEBUG( "sifs_open: open syscall failed\n" );
			if( errno == ENAMETOOLONG ) {
				status = SIFS_ENAMETOOLONG;
				goto out;
			}

			if( errno == ENOENT || errno == ENOTDIR || errno == EISDIR ) {
				status = SIFS_ENOENT;
				goto out;
			}

			if( errno == EACCES  || errno == EROFS ) {
				status = SIFS_EACCES;
				goto out;
			}

			if( errno == ENXIO || errno == ETXTBSY || errno == ENODEV ) {
				status = SIFS_ETEMP;
				goto out;
			}

			status = SIFS_ESERVER;
			goto out;
		}
SIFS_PROFILE( "sifs_open: open syscall succeeded\n" );

		if( desc_insert( path, fd ) < 0 ) {
			SIFS_DEBUG( "sifs_open: Inserting file descriptor into hash table failed\n" );
			status = SIFS_ESERVER;
			goto out;
		}
SIFS_PROFILE( "sifs_open: Inserting file descriptor into hash table succeeded\n" );
	}

	if( desc_increase( path ) < 0 ) {
		SIFS_DEBUG( "sifs_open: Increasing reference counter failed\n" );
		status = SIFS_ESERVER;
	}
SIFS_PROFILE( "sifs_open: Inserting reference counter succeeded\n" );

out:
	PUT_32BIT( conn->out.buffer + 4, 0 );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_open: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_open: Sending reply to client succeeded\n" );

	return 0;
}
