/*
 * Returns the contents of a link
 */



#include "sifs-process.h"



int sifs_readlink( struct connection* conn )
{
	int len = 0;
	int linklen = 0;
	ssize_t size = 0;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int8_t linkbuf[1024];
	u_int32_t packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_readlink: Read link content:\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1028 ) {   // one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_readlink: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_readlink: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_readlink: Reading request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_readlink: Receiving request payload succeeded\n" );

	if( conn->mntpath[0] == 0 ) {   // No filesystem is mounted
		SIFS_DEBUG( "sifs_readlink: There was no mount request before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: Working directory is correct\n" );

	if( GET_16BIT( conn->in.buffer ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_readlink: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer +2, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_readlink: Decoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	if( ( linklen = readlink( path, linkbuf, 1023 ) ) < 0 ) {
		SIFS_DEBUG( "sifs_readlink: readlink syscall failed\n" );
		if( errno == ENAMETOOLONG ) {
			status = SIFS_ENAMETOOLONG;
			goto out;
		}

		if( errno == ENOENT || errno == ENOTDIR ) {
			status = SIFS_ENOENT;
			goto out;
		}

		if( errno == EACCES ) {
			status = SIFS_EACCES;
			goto out;
		}

		if( errno == EINVAL ) {
			status = SIFS_EINVAL;
			goto out;
		}

		if( errno == EIO ) {
			status = SIFS_EIO;
			goto out;
		}

		status = SIFS_ESERVER;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: readlink syscall succeeded\n" );

	linkbuf[linklen] = '\0';
	PUT_16BIT( conn->out.buffer + 12, 1 );

	if( ( len= str_encode( conn->out.buffer +14, linkbuf ) ) < 0 ) {
		SIFS_DEBUG( "sifs_readlink: Encoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_readlink: Encoding string succeeded, path = " );
SIFS_PROFILE( linkbuf );
SIFS_PROFILE( "\n" );

	size = 2 + len;

out:
	PUT_32BIT( conn->out.buffer + 4, size );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, size + 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_readlink: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_readlink: Sending reply to client succeeded\n" );

	return 0;
}
