/*
 * Close the file
 */



#include "sifs-process.h"



int sifs_release( struct connection* conn )
{
	int fd = 0;
	int len = 0;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int32_t  packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_release: Release file:\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1028 ) {   // one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_release: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_release: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_release: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Reading request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_release: Receiving request payload failed\n" );

	if( conn->mntpath[0] == 0 ) {   // No filesystem is mounted
		SIFS_DEBUG( "sifs_release: There was no mount request before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_release: Working directory is correct\n" );

	if( GET_16BIT( conn->in.buffer ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_release: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_release: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer + 2, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Decoding string failed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_release: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	if( ( fd = desc_decrease( path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Decreasing reference counter failed\n" );
		status = SIFS_EOK;   // After a reconnect we have no info about previously opened files
		goto out;
	}
SIFS_PROFILE( "sifs_release: Deserting reference counter succeeded\n" );

	if( fd == 1 ) {
		if( close( fd ) < 0 ) {
			SIFS_DEBUG( "sifs_release: close syscall failed\n" );
			switch( errno ) {
				case EINTR:
					while( errno == EINTR ) {
						close( fd );
					}
					break;
				case EIO:
					status = SIFS_EIO;
					goto out;
					break;
				default:
					status = SIFS_ESERVER;
					goto out;
			}
		}
SIFS_PROFILE( "sifs_release: close syscall succeeded\n" );

		if( desc_remove( path ) < 0 ) {
			SIFS_DEBUG( "sifs_release: Removing file descriptor from hash table failed\n" );
			status = SIFS_ESERVER;
			goto out;
		}
SIFS_PROFILE( "sifs_release: Removing file descriptor from hash table succeeded\n" );
	}

out:
	PUT_32BIT( conn->out.buffer + 4, 0 );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_release: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_release: Sending reply to client succeeded\n" );

	return 0;
}
