/*
 *	SIFS OpenSSH server module
 */



#include "sifs-server.h"
#include "sifs-process.h"



void closefd( struct connection* conn )
{
	close( conn->in.fd );
	close( conn->out.fd );
}


int process( struct connection* conn )
{
	int retval = 0;
	int opc = GET_32BIT( conn->in.buffer + 8 );

	switch( opc ) {
		case SIFS_MOUNT:
			retval = sifs_mount( conn );
			break;
		case SIFS_UNMOUNT:
			retval = sifs_unmount( conn );
			break;
		case SIFS_STATFS:
			retval = sifs_statfs( conn );
			break;
		case SIFS_OPEN:
			retval = sifs_open( conn );
			break;
		case SIFS_CREATE:
			retval = sifs_create( conn );
			break;
		case SIFS_MKDIR:
			retval = sifs_mkdir( conn );
			break;
		case SIFS_MKNOD:
			retval = sifs_mknod( conn );
			break;
		case SIFS_RELEASE:
			retval = sifs_release( conn );
			break;
		case SIFS_UNLINK:
			retval = sifs_unlink( conn );
			break;
		case SIFS_RMDIR:
			retval = sifs_rmdir( conn );
			break;
		case SIFS_LINK:
			retval = sifs_link( conn );
			break;
		case SIFS_SYMLINK:
			retval = sifs_symlink( conn );
			break;
		case SIFS_READLINK:
			retval = sifs_readlink( conn );
			break;
		case SIFS_RENAME:
			retval = sifs_rename( conn );
			break;
		case SIFS_TRUNCATE:
			retval = sifs_truncate( conn );
			break;
		case SIFS_GETATTR:
			retval = sifs_getattr( conn );
			break;
		case SIFS_SETATTR:
			retval = sifs_setattr( conn );
			break;
		case SIFS_READDIR:
			retval = sifs_readdir( conn );
			break;
		case SIFS_READ:
			retval = sifs_read( conn );
			break;
		case SIFS_WRITE:
			retval = sifs_write( conn );
			break;
		case SIFS_FSYNC:
			retval = sifs_fsync( conn );
			break;
		case SIFS_LOCK:
			retval = sifs_lock( conn );
			break;
		default:
			retval = sifs_invalidcmd( conn );
			break;
	}

	return retval;
}



int main( int argc, char* argv[] )
{
	ssize_t rcount = 0;
	struct connection conn;


	memset( &conn, 0, sizeof( struct connection ) );

	if( ( conn.in.fd = dup( STDIN_FILENO ) ) < 0 ) {
		SIFS_DEBUG( "Unable to duplicate stdin" );
		exit( 1 );
	}

	if( ( conn.out.fd = dup( STDOUT_FILENO ) ) < 0 ) {
		SIFS_DEBUG( "Unable to duplicate stdout" );
		exit( 1 );
	}

#ifdef HAVE_CYGWIN
	setmode( conn.in.fd, O_BINARY );
	setmode( conn.out.fd, O_BINARY );
#endif

	while( ( rcount = atomicio( read, conn.in.fd, conn.in.buffer, 12 ) ) == 12 ) {
SIFS_PROFILE( "main: Read request header\n" );
		if( process( &conn ) < 0 ) {
			SIFS_DEBUG( "main: Processing request failed\n" );
			goto out;
		}
SIFS_PROFILE( "main: Processing request succeeded\n" );
	}

	if( rcount < 0 ) {
		SIFS_DEBUG( "main: Unable to receive message from client" );
	}

out:
	closefd( &conn );
SIFS_PROFILE( "main: All file descriptors closed\n" );
}
