/*
 * Clean up and terminate
 */



#include "sifs-process.h"



int sifs_unmount( struct connection* conn )
{
	int len = 0;
	char* path = NULL;
	int status = SIFS_EOK;
	u_int32_t packetlen = GET_32BIT( conn->in.buffer + 4 );


SIFS_PROFILE( "sifs_unmount: Release previously mounted directory:\n" );

	memcpy( conn->out.buffer, conn->in.buffer, 4 );

	if( packetlen > 1028 ) {   // one string
		if( consume_packet( conn->in.fd, conn->in.buffer, MAXBUFSIZE, packetlen ) < 0 ) {
			SIFS_DEBUG( "sifs_unmount: Unable to consume packet\n" );
			return -1;
		}
		SIFS_DEBUG( "sifs_unmount: Packet is longer than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_unmount: Request payload length is ok\n" );

	if( atomicio( read, conn->in.fd, conn->in.buffer, packetlen ) < 0 ) {
		SIFS_DEBUG( "sifs_unmount: Receiving request payload from client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_unmount: Receiving request payload succeeded\n" );

	if( conn->mntpath[0] == 0 ) {   // No filesystem is mounted
		SIFS_DEBUG( "sifs_unmount: A filesystem was mounted before\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_unmount: Request was first request\n" );

	if( GET_16BIT( conn->in.buffer ) != 1 ) {   // More than one string
		SIFS_DEBUG( "sifs_unmount: Request contains more strings than allowed\n" );
		status = SIFS_EPROTO;
		goto out;
	}
SIFS_PROFILE( "sifs_unmount: Number of request strings is ok\n" );

	if( ( len = str_decode( conn->in.buffer + 2, &path ) ) < 0 ) {
		SIFS_DEBUG( "sifs_unmount: Decoding string failed\n" );
		status = len;
		goto out;
	}
SIFS_PROFILE( "sifs_unmount: Decoding string succeeded, path = " );
SIFS_PROFILE( path );
SIFS_PROFILE( "\n" );

	if( strcmp( conn->mntpath, path ) != 0 ) {
		SIFS_DEBUG( "sifs_unmount: Unmount path is invalid\n" );
		status = SIFS_ENOENT;
	}
SIFS_PROFILE( "sifs_unmount: Unmount path is valid\n" );

out:
	PUT_32BIT( conn->out.buffer + 4, 0 );
	PUT_32BIT( conn->out.buffer + 8, status );

	if( atomicio( write, conn->out.fd, conn->out.buffer, 12 ) < 0 ) {
		SIFS_DEBUG( "sifs_unmount: Sending reply to client failed\n" );
		return -1;
	}
SIFS_PROFILE( "sifs_unmount: Sending reply to client succeeded\n" );

	return 0;
}
